var currency = {}


// 通用点击方法

/**
 * 节点点击
 * @param {*} nodeinfo 节点id，或desc信息。
 * @param {*} nodetype 节点类型：d，t，i，tc，dc。
 * @param {*} clicktype 点击类型：1->节点点击，2->绑定坐标点击。
 * @param {*} sleeptime 数组，一个元素固定时间，两个元素随机区间。单位毫秒
 * @returns 点击是否成功。
 */
currency.nodeClick = function (nodeinfo, nodetype, clicktype, sleeptime) {
    let node;
    switch (nodetype) {
        case "tc":
            node = textContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().findOnce();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().findOnce();
            break;
        case "i":
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
        default:
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
    }
    if (node) {
        if (clicktype === 1) {
            node.click();
        } else {
            click(node.bounds().centerX(), node.bounds().centerY());
        }
        if (sleeptime.length === 1) {
            sleep(sleeptime[0]);
        } else if (sleeptime.length === 2) {
            sleep(random(sleeptime[0], sleeptime[1]));
        }
        return true;
    } else {
        return false;
    }
};


/**
 * 对集合元素点击的方法
 * @param {*} nodeinfo 节点id
 * @param {*} nodetype 节点类型
 * @param {*} sleeptime 数组
 * @param {*} callback 回调函数
 */
currency.itemClick = function (nodeinfo, nodetype, sleeptime, callback) {
    let node;
    switch (nodetype) {
        case "tc":
            node = textContains(nodeinfo).visibleToUser().find();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().find();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().find();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().find();
            break;
        case "i":
            node = id(nodeinfo).visibleToUser().find();
            break;
        default:
            node = id(nodeinfo).visibleToUser().find();
            break;
    }
    if (node && node.length != 0) {
        node.forEach(element => {
            element.click();

            if (sleeptime.length === 1) {
                sleep(sleeptime[0]);
            } else if (sleeptime.length === 2) {
                sleep(random(sleeptime[0], sleeptime[1]));
            }

            callback && callback();

        });
    }
}
/**
 * 对集合元素坐标点击的方法
 * @param {*} nodeinfo 节点id
 * @param {*} nodetype 节点类型
 * @param {*} sleeptime 数组
 * @param {*} callback 回调函数
 */
 currency.XYitemClick = function (nodeinfo, nodetype, sleeptime, callback) {
    let node;
    switch (nodetype) {
        case "tc":
            node = textContains(nodeinfo).visibleToUser().find();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().find();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().find();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().find();
            break;
        case "i":
            node = id(nodeinfo).visibleToUser().find();
            break;
        default:
            node = id(nodeinfo).visibleToUser().find();
            break;
    }
    if (node && node.length != 0) {
        node.forEach (function(element, index) {
            if(index != node.length - 1){
                click(element.bounds().centerX(), element.bounds().centerY());

                if (sleeptime.length === 1) {
                    sleep(sleeptime[0]);
                } else if (sleeptime.length === 2) {
                    sleep(random(sleeptime[0], sleeptime[1]));
                }

                callback && callback();
            }    
        });
    }
}

// 其他通用方法

/**
 * 检测到节点后向下滑动 直播间跳过
 * @param {*} nodeinfo 节点信息
 * @param {*} nodetype 节点类型
 * @param {*} sleeptime 数组
 * @returns 
 */
currency.detectSlide = function (nodeinfo, nodetype, sleeptime) {
    let node;
    switch (nodetype) {
        case "tc":
            node = textContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().findOnce();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().findOnce();
            break;
        case "i":
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
        default:
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
    }
    if (node) {
        swipe(device.width * 0.75, device.height * 0.7, device.width * 0.8, device.height * 0.1, 300);
        if (sleeptime.length === 1) {
            sleep(sleeptime[0]);
        } else if (sleeptime.length === 2) {
            sleep(random(sleeptime[0], sleeptime[1]));
        }
        return true;
    } else {
        return false;
    }
}

/**
 * 概率处理
 * @param {*} c 字符串 带百分号
 * @returns 
 */
currency.chance = function (c) {
    switch (c) {
        case "100%":
            return true;
        case "0%":
            return false;
        default:
            return (random(1, 100) <= parseInt(c.replace("%", "")));
    }
}

/**
 * 检查应用版本
 * @param {*} pkginfo 应用包名
 * @param {*} sup_version 支持版本号
 */
currency.checkApp = function (pkginfo, sup_version) {
    let app_version;
    try {
        let pckMan = context.getPackageManager();
        let packageInfo = pckMan.getPackageInfo(pkginfo, 0);
        app_version = packageInfo.versionName;
    } catch (e) {
        app_version = "0.0.0";
    }
    if (app_version != sup_version) {
        dialogs.build({
            title: "提示",
            content: "当前应用版本" + app_version + "不正确",
            positive: "确认",
            cancelable: false,
        }).on("positive", function () {
            toast("程序停止");
            engines.stopAll();
            exit();
        }).show();
    }
}

/**
 * 循环返回，页面定位
 * @param {*} nodeinfo 节点信息
 * @param {*} nodetype 节点类型
 * @param {*} positiontype 定位类型，1->节点存在执行返回，2->节点不存在执行返回。
 * @param {*} sleeptime 数组
 */
currency.loopBack = function (nodeinfo, nodetype, positiontype, sleeptime) {
    let node;
    let result;
    while (true) {
        switch (nodetype) {
            case "tc":
                node = textContains(nodeinfo).visibleToUser().findOnce();
                break;
            case "dc":
                node = descContains(nodeinfo).visibleToUser().findOnce();
                break;
            case "t":
                node = text(nodeinfo).visibleToUser().findOnce();
                break;
            case "d":
                node = desc(nodeinfo).visibleToUser().findOnce();
                break;
            case "i":
                node = id(nodeinfo).visibleToUser().findOnce();
                break;
            default:
                node = id(nodeinfo).visibleToUser().findOnce();
                break;
        }

        if (positiontype === 1) {
            result = node;
        } else {
            result = !node;
        }

        if (result) {
            toastLog("返回")
            back();
        } else {
            break;
        }

        if (sleeptime.length === 1) {
            sleep(sleeptime[0]);
        } else if (sleeptime.length === 2) {
            sleep(random(sleeptime[0], sleeptime[1]));
        }
    }

}

/*
通过节点高度滑动
@ nodeinfo    节点信息
@ nodetype    节点类型
@ sleeptime    休眠时间
@ num          次数
*/
currency.boundsSlide = function (nodeinfo, nodetype, sleeptime) {
    let node;
    switch (nodetype) {
        case "i":
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().findOnce();
            break;
        case "tc":
            node = textContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().findOnce();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().findOnce();
            break;
    }

    if (node) {
        let hd_h = node.bounds();
        let x1 = 300;
        let y1 = hd_h.bottom;

        let x2 = 300;
        let y2 = hd_h.top;

        gesture(800, [x1, y1 - 250], [x2, y2]);

        if (sleeptime.length == 1) {
            sleep(sleeptime[0]);
        } else {
            sleep(random(sleeptime[0], sleeptime[1]));
        }
    } else {
        return false;
    }
}

/*
全屏滑动
*/
currency.fullSlide = function () {
    let W = device.width;
    let H = device.height;
    gesture(300, [W * 0.8, H * 0.7], [W * 0.85, 0]);
    sleep(3000);
}

/**
 * 取节点的描述信息
 * @param {*} nodeinfo 节点信息
 * @param {*} resulttype t->取text，d->取desc
 * @returns 
 */
currency.getContent = function (nodeinfo, resulttype) {
    let result = "";
    let node;
    node = id(nodeinfo).visibleToUser().findOnce();
    if (node) {
        if (resulttype === "t") {
            result = node.text();
        } else if (resulttype === "d") {
            result = node.desc();
        }
    }
    return result;
}

/**
 * 消息发送，直播间或评论发表
 * @param {*} inputnode 输入框节点，可点击
 * @param {*} messageinfo 要发送的信息
 * @param {*} sendnode 发送按钮节点，可点击
 */
currency.sendMsg = function (inputnode, message, sendnode) {
    let input;
    let send;
    let result = false;
    if (Array.isArray(message) && message.length > 0) {
        message.forEach((element) => {
            input = id(inputnode).visibleToUser().findOnce(); //输入框
            if (input) {
                input.click();
                sleep(random(2500, 3000));
                setText(element);
                sleep(random(2500, 3000));
                send = id(sendnode).visibleToUser().findOnce(); //发送
                if (send) {
                    send.click();
                    sleep(random(2500, 3000));
                    result = true;
                }
            }
        });

    }
    return result;

}

/**
 * 话术处理
 * @param {*} words 话术
 * @returns 返回数组，根据数组长度处理，或使用foreach方法
 */
currency.randomWords = function (words) {
    let words_;
    let result = [];
    if (Array.isArray(words) && words.length > 0) {
        words_ = words[random(0, words.length - 1)]
        if (words_.indexOf("*") >= 0) {
            words_ = words_.split("*");
            result = words_;
        } else {
            result.push(words_);
        };
    };
    log("randomwords_result---" + result)
    return result;
}

/**
 * 检测当前页面节点是否存在
 * @param {*} nodeinfo 
 * @param {*} nodetype 
 * @returns 
 */
currency.nodeExists = function (nodeinfo, nodetype) {
    let node;
    switch (nodetype) {
        case "tc":
            node = textContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "dc":
            node = descContains(nodeinfo).visibleToUser().findOnce();
            break;
        case "t":
            node = text(nodeinfo).visibleToUser().findOnce();
            break;
        case "d":
            node = desc(nodeinfo).visibleToUser().findOnce();
            break;
        case "i":
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
        default:
            node = id(nodeinfo).visibleToUser().findOnce();
            break;
    }
    if (node) {
        return true;
    } else {
        return false;
    }
}


/**
 * 获取粉丝数
 * @param {*} nodeinfo 
 */
currency.getFans = function (nodeinfo) {
    let node;
    let content, contenta;
    node = id(nodeinfo).visibleToUser().findOnce();
    if (node) {
        content = node.text();
        if (content.indexOf("w") >= 0) {
            contenta = parseFloat(content);
            contenta = contenta * 10000;
            contenta = parseInt(contenta);
            content = contenta;

        } else if (content.indexOf("万") >= 0) {
            contenta = parseFloat(content);
            contenta = contenta * 10000;
            contenta = parseInt(contenta);
            content = contenta;
        }
        content = parseInt(content);
        log("contenta------" + content);
        return content;
    } else {
        return 0;
    }
}


/**
 * 判断是否在区间内
 * @param {*} str 
 * @param {*} minnum 最小值
 * @param {*} maxnum 最大值
 * @returns 
 */
currency.isRangeIn = function (str, minnum, maxnum) {
    var num = parseFloat(str);

    if (num <= maxnum && num >= minnum) {
        return true;
    }

    return false;

};

module.exports = currency;